#!/usr/bin/ruby
#
#   send-message.rb
#
# DESCRIPTION:
#   Sensu Messages Handler For Custom Messages gateway
#
# OUTPUT:
#   None unless there is an error
#
# PLATFORMS:
#   Linux
#
# DEPENDENCIES:
#   gem: sensu-handler
#
# USAGE:
#
#   Configure your messages web_hook, auth_token, recipient and from here:
#     messages.json
#
# NOTES:
#
# LICENSE:
#   Copyright (c) Vaimo Group. All rights reserved.
#   See LICENSE_VAIMO.txt for license details.
#
require 'json'
require 'net/http'
require 'net/https'
require 'uri'
require 'logger'

def sensu_config_files
  ['/etc/sensu/config.json'] + Dir['/etc/sensu/conf.d/**/*.json']
end

def sensu_load_config(filename)
  JSON.parse(File.open(filename, 'r').read)
rescue
  {}
end

def sensu_settings
  @sensu_settings ||= sensu_config_files.map { |f| sensu_load_config(f) }.reduce do |a, b|
    sensu_deep_merge(a, b)
  end
end

def sensu_setting(key)
  @sensu_settings[key]
end

def sensu_deep_merge(hash_one, hash_two)
  merged = hash_one.dup
  hash_two.each do |key, value|
    merged[key] = if hash_one[key].is_a?(Hash) && value.is_a?(Hash)
      sensu_deep_merge(hash_one[key], value)
    elsif hash_one[key].is_a?(Array) && value.is_a?(Array)
      hash_one[key].concat(value).uniq
    else
      value
    end
  end
  merged
end

def messages_key_value(args, key, default)
  args[key] || default
end

def messages_post_args(args, settings)
  {
      auth_token: settings['auth_token'],
      receiver:   settings['recipient'],
      from:       settings['from'],
      server:     messages_key_value(args, :server, nil),
      level:      messages_key_value(args, :level, nil),
      message:    messages_key_value(args, :message, nil),
      component:  messages_key_value(args, :component, nil),
      output:     messages_key_value(args, :output, nil),
      history:    messages_key_value(args, :history, nil),
      event:      messages_key_value(args, :event, nil),
  }
end

def messages_parse_response(response)
  data = JSON.parse(response)

  hash = {}
  data.each do |key, value|
    hash[key] = value
  end

  raise "Messages WebHook: #{hash['message']}" if hash['error']

  hash
end

def messages_level(status)
  case status
    when 0
      'normal'
    when 1
      'warning'
    when 2
      'error'
    else
      'unknown'
  end
end

def messages_send(post_args, settings)
  header = { 'Content-Type': 'text/json' }

  return unless settings['webhook_url']

  uri = URI.parse(settings['webhook_url'])

  http         = Net::HTTP.new(uri.host, uri.port)
  http.use_ssl = true

  request      = Net::HTTP::Post.new(uri.request_uri, header)
  request.body = post_args.to_json

  response = http.request(request)

  messages_parse_response(response.body)
end

log       = Logger.new('/tmp/sensu-messages-handler.log')
log.level = Logger::INFO

begin
  raw_data = STDIN.read
  event    = JSON.parse(raw_data)

  sensu_settings
  messages_settings = sensu_setting('messages')

  args = {
      server:    event['client']['name'],
      component: event['check']['name'],
      level:     messages_level(event['check']['status']),
      output:    event['check']['output'],
      history:   event['check']['history'],
      event:     event,
  }

  post_args = messages_post_args(args, messages_settings)

  messages_send(post_args, messages_settings)
rescue => e
  log.error "Messages Error: #{e}"
  log.error "Messages Trace: #{e.backtrace}"
end